package core.api.controller;

import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.history.TaskgroupActivityInfo;
import core.api.models.timemanager.history.WorkingStatus;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.Taskgroup;
import core.services.PermissionResult;
import core.services.ServiceExitCode;
import core.services.TaskScheduleService;
import core.services.TaskgroupService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class StatisticController {

    @Autowired private TaskScheduleService taskScheduleService;
    @Autowired private TaskgroupService taskgroupService;

    @GetMapping("/history/workingStatus")
    public ResponseEntity<?> getWorkingStatus() {
        boolean missedSchedules = !taskScheduleService.getAllMissedSchedulesOfUser(SecurityContextHolder.getContext().getAuthentication().getName()).isEmpty();
        List<AbstractSchedule> executedSchedules = taskScheduleService.getActiveSchedulesOfToday(SecurityContextHolder.getContext().getAuthentication().getName());
        int activeTime = 0;
        for(AbstractSchedule schedule : executedSchedules) {
            if(schedule.getStartTime().toLocalDate().isBefore(LocalDate.now())) {
                activeTime += (int) Duration.between(LocalDateTime.of(LocalDate.now(), LocalTime.MIDNIGHT), schedule.getStopTime()).toMinutes();
            } else {
                activeTime += (int) Duration.between(schedule.getStartTime(), schedule.getStopTime()).toMinutes();
            }
        }

        return ResponseEntity.ok(new WorkingStatus(missedSchedules, activeTime));
    }

    @GetMapping("/statistics/taskgroup-activity/{taskgroupID}/{startingDate}/{endingDate}/{includeSubTaskgroups}")
    public ResponseEntity<?> getTaskgroupActivity(@PathVariable long taskgroupID, @PathVariable String startingDate, @PathVariable String endingDate, @PathVariable boolean includeSubTaskgroups){
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(taskgroupPermissionResult.isNoPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        } else if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        List<TaskgroupActivityInfo> activityInfos = taskgroupPermissionResult.getResult().calcActivityInfo(includeSubTaskgroups,
                LocalDate.parse(startingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd")), LocalDate.parse(endingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        activityInfos.sort(new Comparator<TaskgroupActivityInfo>() {
            @Override
            public int compare(TaskgroupActivityInfo o1, TaskgroupActivityInfo o2) {
                return o1.getDate().compareTo(o2.getDate());
            }
        });
        return ResponseEntity.ok(activityInfos);
    }

    @GetMapping("/history/schedules/{date}")
    public ResponseEntity<?> getPastSchedules(@PathVariable String date) {
        LocalDate dateArg = LocalDate.parse(date, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        List<AbstractSchedule> abstractSchedules = taskScheduleService.findSchedulesByDate(SecurityContextHolder.getContext().getAuthentication().getName(), dateArg);
        return ResponseEntity.ok(abstractSchedules.stream().map(AbstractSchedule::toScheduleInfo).toList());
    }
}
